/***************************************************************************
                          photoalbum.cpp  -  description
                             -------------------
    begin                : Mon Jan 28 2002
    copyright            : (C) 2002 by Graham Morrison
    email                : kalbum@paldandy.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kpixmap.h>
#include <qimage.h>
#include <qfileinfo.h>

#include <kglobal.h>
#include <kstddirs.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kurllabel.h>
#include <kurlrequester.h>
#include <kimageeffect.h>
#include <klocale.h>
#include <qpixmap.h>
#include <qdatastream.h>

#include <kfilemetainfo.h>
#include <qstringlist.h>

#include "photoalbum.h"
#include "kalbumnoimage.h"

#define PHTHMSZX	110
#define PHTHMSZY	 90

#define MAX(A,B) (((A)>(B)) ? (A) : (B))

PhotoAlbum::PhotoAlbum() {

    ignoredate = false;
    ignoretime = false;

}


PhotoAlbum::PhotoAlbum(QDataStream & stream, int version) {

    QString str_title, str_comment, str_fileurl, str_date;
    int year, month, day, hour, minute, second;
    Q_INT32 pix_xsize, pix_ysize;
    Q_INT32 pix_ignoredate, pix_rotate, pix_ignoretime;
//    KPixmap pix_thumbnail;

    version = 0;

    stream>>str_title;
    stream>>str_comment;
    stream>>str_fileurl;
    stream>>year;
    stream>>month;
    stream>>day;
    stream>>hour;
    stream>>minute;
    stream>>second;
    stream>>pix_rotate;
    stream>>pix_xsize;
    stream>>pix_ysize;
    stream>>pix_ignoredate;
    stream>>pix_ignoretime;
//    stream>>pix_thumbnail;

    title 	= str_title;
    comment = str_comment;
    fileurl = str_fileurl;
    date.setYMD(year, month, day);
    time.setHMS(hour, minute, second);
    rotate = pix_rotate;
    xsize = pix_xsize;
    ysize = pix_ysize;
    ignoredate = pix_ignoredate;
    ignoretime = pix_ignoretime;

//    thumbnail = pix_thumbnail;

}


void PhotoAlbum::writeToStream(QDataStream & stream) const {
    // Don't forget to add written elements to the copy() function

    stream<<title;
    stream<<comment;
    stream<<fileurl.url();
    stream<<date.year();
    stream<<date.month();
    stream<<date.day();
    stream<<time.hour();
    stream<<time.minute();
    stream<<time.second();
    stream<<(Q_INT32)rotate;
    stream<<(Q_INT32)xsize;
    stream<<(Q_INT32)ysize;
    stream<<(Q_INT32)ignoredate;
    stream<<(Q_INT32)ignoretime;
//    stream<<thumbnail;
}

//PhotoAlbum::readFromStream(QDataStream & stream) const;


PhotoAlbum *PhotoAlbum::copy() {

    PhotoAlbum *photo = new PhotoAlbum;
    ;

    photo->fileurl 	= this->fileurl;
    photo->filesize 	= this->filesize;
    photo->date		= this->date;
    photo->time		= this->time;
    photo->thumbnail    = this->thumbnail;
    photo->selected	= this->selected;
    photo->comment	= this->comment;
    photo->title	= this->title;
    photo->iconitem	= NULL;
    photo->rotate       = this->rotate;
    photo->xsize        = this->xsize;
    photo->ysize        = this->ysize;
    photo->ignoredate   = this->ignoredate;
    photo->ignoretime   = this->ignoretime;

    return photo;

}


PhotoAlbum::~PhotoAlbum() {}



PhotoAlbum *PhotoAlbum::dataPhoto() {

    QString tmpfile;
    QString dir;
    int extpos;

    QString test;
    QStringList exif_list;

    selected = true;

    KIO::NetAccess::download( fileurl, tmpfile, NULL);
	KFileMetaInfo exif_info = KFileMetaInfo(tmpfile);
	exif.Data(exif_info);

    if (fileurl.isValid()) {

    	// Transform exif values into kalbum useable values
    	exifTransform();

        updateThumbnail();

        QFileInfo finfo(tmpfile);

        filesize = finfo.size();

        if (!date.isValid()){
        	if (exif.creation_date.isValid()){
            	date = exif.creation_date;
            } else {
            	date = finfo.lastModified().date();
            }
        }

        if (time.isNull()){
        	if (exif.creation_time.isValid()){
            	time = exif.creation_time;
            } else {
	        	time = finfo.lastModified().time();
            }
        }

        filename = fileurl.fileName();
        extpos=filename.find(".");

        if (extpos)
            filename=filename.remove(extpos, (filename.length()-extpos));

        if (title.isEmpty())
            title=filename;

        if (comment.isEmpty()){
        	if (exif.comment.isEmpty()){
            	comment = exif.comment;
            } else {
            	comment = filename;
            }
        }

        KIO::NetAccess::removeTempFile( tmpfile );
        return this;
    }

    return NULL;

}

void PhotoAlbum::exifTransform() {

	switch (exif.orientation) {
        case 3:
            rotate = 180;
            break;
        case 6:
            rotate = 90;
            break;
        case 8:
            rotate = 270;
            break;
        default:
            rotate = 0;
    	break;
	}
}


QString PhotoAlbum::textEncode() {
    QString buffer;

    buffer = "Title:" + this->title + ":Title\n";
    buffer = buffer + "Date:" + this->date.toString() + ":Date\n";
    buffer = buffer + "URL:" + this->fileurl.path() + ":URL\n";
    buffer = buffer + "Comment:" + this->comment + ":Comment\n";

    return buffer;

}


PhotoAlbum *PhotoAlbum::textDecode(QString &buffer, int item) {
    int start, end;

    start=buffer.find("URL:", item);

    if (start) {
        end=buffer.find(":URL", item);
        this->title=buffer.mid(start, (end-start));
    } else
        return NULL;

    return this;
}

void PhotoAlbum::removePhoto(const KURL& url) {


}



KPixmap PhotoAlbum::loadSlide() {

	QString dir;
	KPixmap background;
			
	KGlobal::dirs()->addResourceType("kalbum_data", KGlobal::dirs()->kde_default("data")
                        	                 + "kalbum/data/");

	dir = KGlobal::dirs()->findResourceDir("kalbum_data", "slides/slide.png");
	dir = dir + "slides/slide.png";

	background.load(dir);
	
	return background;
			

}
			

void PhotoAlbum::updateThumbnail() {

    if (thumbnail.isNull())
   	thumbnail = loadSlide();

}


